@@ -493,16 +493,16 @@ char *browse(char *path)
 #endif
 		function = interpret(kbinput);
 
-		if (function == full_refresh || function == do_help) {
+		if (function == do_help || function == full_refresh) {
 			function();
 #ifndef NANO_TINY
 			/* Simulate a terminal resize to force a directory reread,
 			 * or because the terminal dimensions might have changed. */
-			kbinput = KEY_WINCH;
+			kbinput = THE_WINDOW_RESIZED;
 		} else if (function == do_toggle && get_shortcut(kbinput)->toggle == NO_HELP) {
 			TOGGLE(NO_HELP);
 			window_init();
-			kbinput = KEY_WINCH;
+			kbinput = THE_WINDOW_RESIZED;
 #endif
 		} else if (function == do_search_backward) {
 			search_filename(BACKWARD);
@@ -572,8 +572,7 @@ char *browse(char *path)
 
 			/* If the given path is relative, join it with the current path. */
 			if (*path != '/') {
-				path = nrealloc(path, strlen(present_path) +
-												strlen(answer) + 1);
+				path = nrealloc(path, strlen(present_path) + strlen(answer) + 1);
 				sprintf(path, "%s%s", present_path, answer);
 			}
 
@@ -642,7 +641,7 @@ char *browse(char *path)
 			implant(first_sc_for(MBROWSER, function)->expansion);
 #endif
 #ifndef NANO_TINY
-		} else if (kbinput == KEY_WINCH) {
+		} else if (kbinput == THE_WINDOW_RESIZED) {
 			;  /* Gets handled below. */
 #endif
 		} else if (function == do_exit) {
@@ -652,7 +651,7 @@ char *browse(char *path)
 
 #ifndef NANO_TINY
 		/* If the terminal resized (or might have), refresh the file list. */
-		if (kbinput == KEY_WINCH) {
+		if (kbinput == THE_WINDOW_RESIZED) {
 			/* Remember the selected file, to be able to reselect it. */
 			present_name = copy_of(filelist[selected]);
 			goto read_directory_contents;
