@@ -1,7 +1,7 @@
 /**************************************************************************
  *   move.c  --  This file is part of GNU nano.                           *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2023 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2024 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014-2018 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -41,7 +41,7 @@ void to_last_line(void)
 	openfile->placewewant = xplustabs();
 
 	/* Set the last line of the screen as the target for the cursor. */
-	openfile->current_y = editwinrows - 1;
+	openfile->cursor_row = editwinrows - 1;
 
 	refresh_needed = TRUE;
 #ifdef ENABLE_COLOR
@@ -124,7 +124,7 @@ void do_page_up(void)
 	if (ISSET(JUMPY_SCROLLING)) {
 		openfile->current = openfile->edittop;
 		leftedge = openfile->firstcolumn;
-		openfile->current_y = 0;
+		openfile->cursor_row = 0;
 		target_column = 0;
 	} else
 #endif
@@ -156,7 +156,7 @@ void do_page_down(void)
 	if (ISSET(JUMPY_SCROLLING)) {
 		openfile->current = openfile->edittop;
 		leftedge = openfile->firstcolumn;
-		openfile->current_y = 0;
+		openfile->cursor_row = 0;
 		target_column = 0;
 	} else
 #endif
@@ -176,6 +176,61 @@ void do_page_down(void)
 	refresh_needed = TRUE;
 }
 
+#ifndef NANO_TINY
+/* Place the cursor on the first row in the viewport. */
+void to_top_row(void)
+{
+	size_t leftedge, offset;
+
+	get_edge_and_target(&leftedge, &offset);
+
+	openfile->current = openfile->edittop;
+	leftedge = openfile->firstcolumn;
+
+	set_proper_index_and_pww(&leftedge, offset, FALSE);
+
+	refresh_needed = (openfile->mark != NULL);
+}
+
+/* Place the cursor on the last row in the viewport, when possible. */
+void to_bottom_row(void)
+{
+	size_t leftedge, offset;
+
+	get_edge_and_target(&leftedge, &offset);
+
+	openfile->current = openfile->edittop;
+	leftedge = openfile->firstcolumn;
+
+	go_forward_chunks(editwinrows - 1, &openfile->current, &leftedge);
+	set_proper_index_and_pww(&leftedge, offset, TRUE);
+
+	refresh_needed = (openfile->mark != NULL);
+}
+
+/* Put the cursor line at the center, then the top, then the bottom. */
+void do_cycle(void)
+{
+	if (cycling_aim == 0)
+		adjust_viewport(CENTERING);
+	else {
+		openfile->cursor_row = (cycling_aim == 1) ? 0 : editwinrows - 1;
+		adjust_viewport(STATIONARY);
+	}
+
+	cycling_aim = (cycling_aim + 1) % 3;
+
+	draw_all_subwindows();
+	full_refresh();
+}
+
+/* Scroll the line with the cursor to the center of the screen. */
+void do_center(void)
+{
+	do_cycle();  /* The main loop has set 'cycling_aim' to zero. */
+}
+#endif /* !NANO_TINY */
+
 #ifdef ENABLE_JUSTIFY
 /* Move to the first beginning of a paragraph before the current line. */
 void do_para_begin(linestruct **line)
@@ -526,7 +581,8 @@ void do_up(void)
 
 	set_proper_index_and_pww(&leftedge, target_column, FALSE);
 
-	if (openfile->current_y == 0 && !ISSET(JUMPY_SCROLLING))
+	if (openfile->cursor_row == 0 && !ISSET(JUMPY_SCROLLING) &&
+						(tabsize < editwincols || !ISSET(SOFTWRAP)))
 		edit_scroll(BACKWARD);
 	else
 		edit_redraw(was_current, FLOWING);
@@ -549,7 +605,8 @@ void do_down(void)
 
 	set_proper_index_and_pww(&leftedge, target_column, TRUE);
 
-	if (openfile->current_y == editwinrows - 1 && !ISSET(JUMPY_SCROLLING))
+	if (openfile->cursor_row == editwinrows - 1 && !ISSET(JUMPY_SCROLLING) &&
+								(tabsize < editwincols || !ISSET(SOFTWRAP)))
 		edit_scroll(FORWARD);
 	else
 		edit_redraw(was_current, FLOWING);
@@ -566,7 +623,7 @@ void do_scroll_up(void)
 	if (openfile->edittop->prev == NULL && openfile->firstcolumn == 0)
 		return;
 
-	if (openfile->current_y == editwinrows - 1)
+	if (openfile->cursor_row == editwinrows - 1)
 		do_up();
 
 	if (editwinrows > 1)
@@ -576,7 +633,7 @@ void do_scroll_up(void)
 /* Scroll down one line or chunk without moving the cursor textwise. */
 void do_scroll_down(void)
 {
-	if (openfile->current_y == 0)
+	if (openfile->cursor_row == 0)
 		do_down();
 
 	if (editwinrows > 1 && (openfile->edittop->next != NULL
@@ -587,14 +644,6 @@ void do_scroll_down(void)
 										))
 		edit_scroll(FORWARD);
 }
-
-/* Scroll the line with the cursor to the center of the screen. */
-void do_center(void)
-{
-	adjust_viewport(CENTERING);
-	draw_all_subwindows();
-	full_refresh();
-}
 #endif /* !NANO_TINY || ENABLE_HELP */
 
 /* Move left one character. */
